
#Bee learning code from-
#Bumblebees can detect floral humidity

#Authors:
#Michael J. M. Harrap
#Natalie Hempel de Ibarra
#Henry D. Knowles
#Heather M. Whitney
#Sean A. Rands

#Below is the code for the analysis of the bee learning data for the
#aforementioned publication. Instruction and explanation of code is given in
# sections marked with hashtag (#). Outputs from immediately preceding code are given in
#section marked with inverted commas ('').

#For further explanation of the data, statistical models and the model simplification 
#procedure below see the above publication.

#All analysis was carried out in R version 3.6.3 (2020-02-29) -- "Holding the Windsock"

# clear memory and start afresh
rm(list = ls())

#set working directory to file 'Bee trials Data' file
setwd(".../Bee Trials Data") #replace '...' with your directory

#read learning data
Learning<-read.csv(file = "LearningRateData.csv", header = TRUE)

#convert test group to shorthand title (this is done to condense notation below)
Learning$test<-Learning$test.group

#Specify D and H operator parameters to get desired model comparisons.
#H
Learning$H[Learning$test=="Control"]<-0
Learning$H[Learning$test=="Dry Reward"|Learning$test=="Humid Reward"]<-1
#Note, Ha and Hb are the same and a different operator is not needed
#Ha - is H on it's own (intercept effect), Hb is H:loglast interaction.

#D
Learning$D[Learning$test=="Control"|Learning$test=="Humid Reward"]<-0
Learning$D[Learning$test=="Dry Reward"]<-1

#altH - (Hc in main text) only used if above D is maintained in best model after 1st comparison of dry and humid rewards groups, see main text 
#this allows D vs Control comparison
Learning$Hb[Learning$test=="Control"|Learning$test=="Dry Reward"]<-0
Learning$Hb[Learning$test=="Humid Reward"]<-1

#back and forth test of transformations in datasheet (for peace of mind)
asin(sqrt(0.5))
(sin(0.7853982))^2

#Load packages
library(lme4)

#Below we compare effects of
#(ln transformed) experience 'loglast', Presence of humidity difference 'H' and specific test group 'D'.
#on (asin sqare root transformed) success rate 'ArcCorrect'
#'bee' is included as a random factor


#test effect of random intercepts and random slopes model vs just random intercepts
fullmodel<-lmer(ArcCorrect~loglast+H+D+loglast:H+loglast:D+(loglast|bee), data=Learning, REML = FALSE)

fullmodel2<-lmer(ArcCorrect~loglast+H+D+loglast:H+loglast:D+(1|bee), data=Learning, REML = FALSE)
anova(fullmodel,fullmodel2)

'Data: Learning
Models:
fullmodel2: ArcCorrect ~ loglast + H + D + loglast:H + loglast:D + (1 | bee)
fullmodel: ArcCorrect ~ loglast + H + D + loglast:H + loglast:D + (loglast | 
fullmodel:     bee)
           npar     AIC     BIC logLik deviance  Chisq Df Pr(>Chisq)
fullmodel2    8 -294.32 -264.30 155.16  -310.32                     
fullmodel    10 -291.87 -254.34 155.94  -311.87 1.5475  2     0.4613'

#DeltaAIC
291.87-294.32
'2.45'
#Full model 2 is lower and simpler.
#standing best model is fullmodel2
#individual bees do not have random slopes

#Test for differences between Dry and Humid Reward test group in learning
DropInterD<-lmer(ArcCorrect~loglast+H+D+loglast:H+(1|bee), data=Learning, REML = FALSE)
anova(fullmodel2,DropInterD)
'Data: Learning
Models:
DropInterD: ArcCorrect ~ loglast + H + D + loglast:H + (1 | bee)
fullmodel2: ArcCorrect ~ loglast + H + D + loglast:H + loglast:D + (1 | bee)
           npar     AIC    BIC logLik deviance  Chisq Df Pr(>Chisq)  
DropInterD    7 -290.47 -264.2 152.23  -304.47                       
fullmodel2    8 -294.32 -264.3 155.16  -310.32 5.8559  1    0.01552 *'
290.47-294.32
'3.85'
#fullmodel2 is lower but not to sufficient criteria, doesn't perform better
#but is of better fit...
#Some indication they may differ but not sufficient effect
#DropInterD is simpler and as good

#Test for differences between Dry and Humid Reward test group in intercept (success at 10visits) 
DropD<-lmer(ArcCorrect~loglast+H+loglast:H+(1|bee), data=Learning, REML = FALSE)
anova(DropInterD,DropD)
'Data: Learning
Models:
DropD: ArcCorrect ~ loglast + H + loglast:H + (1 | bee)
DropInterD: ArcCorrect ~ loglast + H + D + loglast:H + (1 | bee)
           npar     AIC     BIC logLik deviance  Chisq Df Pr(>Chisq)
DropD         6 -291.76 -269.25 151.88  -303.76                     
DropInterD    7 -290.47 -264.20 152.23  -304.47 0.7067  1     0.4005'
291.76-290.47
'1.29'
#DropD is lower and simpler

#neither of D's parameters are maintained in best model,
#indicates Dry and Humid rewards test groups share a response

#As neither D parameters are retained we do not need to change H
#or repeat D vs Control group comparisons, we just test this common response
#of humid and dry reward group against the control groups

#Test for differences in Learning between bees presented with humidity differences (Dry and Humid Reward)
DropInterH<-lmer(ArcCorrect~loglast+H+(1|bee), data=Learning, REML = FALSE)
anova(DropD,DropInterH)
'Data: Learning
Models:
DropInterH: ArcCorrect ~ loglast + H + (1 | bee)
DropD: ArcCorrect ~ loglast + H + loglast:H + (1 | bee)
           npar     AIC     BIC logLik deviance Chisq Df Pr(>Chisq)   
DropInterH    5 -283.59 -264.83 146.80  -293.59                       
DropD         6 -291.76 -269.25 151.88  -303.76 10.17  1   0.001428 **'
283.59-291.76
'8.17'
#DropD is lower by sufficient criteria and better fit
#Presence of humidity differences alters learning,
#regardless of whether elevated humidity corresponds with rewards or not.

#Test for differences in intercept between bees presented with humidity differences
DropH<-lmer(ArcCorrect~loglast+loglast:H+(1|bee), data=Learning, REML = FALSE)
anova(DropD,DropH)
'Data: Learning
Models:
DropH: ArcCorrect ~ loglast + loglast:H + (1 | bee)
DropD: ArcCorrect ~ loglast + H + loglast:H + (1 | bee)
      npar     AIC     BIC logLik deviance  Chisq Df Pr(>Chisq)
DropH    5 -293.75 -274.99 151.87  -303.75                     
DropD    6 -291.76 -269.25 151.88  -303.76 0.0128  1       0.91'
293.75-291.76
'1.99'
#DropH is lower AIC and simpler
#intercepts not different enough
#DropH is better

#assessment of learning in control group
DropLast<-lmer(ArcCorrect~loglast:H+(1|bee), data=Learning, REML = FALSE)
anova(DropH,DropLast)
'Data: Learning
Models:
  DropLast: ArcCorrect ~ loglast:H + (1 | bee)
DropH: ArcCorrect ~ loglast + loglast:H + (1 | bee)
npar     AIC     BIC logLik deviance  Chisq Df Pr(>Chisq)   
DropLast    4 -288.67 -273.66 148.34  -296.67                        
DropH       5 -293.75 -274.99 151.87  -303.75 7.0783  1   0.007802 **'
293.75-288.67
'5.08'
#DropH has lower AIC but not to sufficient criteria, but is of better fit
#DropLast is simpler and performs as well
#some indication of background learning but not sufficient effect



#final model indicates control bees do not learn, maintain a near 0.5 success.
#Dry rewards and Humid Rewards groups do not differ from each other, however
#these groups differ in learning from the control group. This means that humidity differences between
#rewarding and nonrewarding flowers allows bees to reach a greater learning success with experience.
#However it doesn't seem to matter whether higher or lower humidity corresponds to rewards
#
#i.e. bees can learn to distinguish flowers that differ in floral humidity but not those that don't.

#final best model is DropLast
#below gives parameter values
#N.B. values produced in 'summary' are still asin-sqrt transformed.
summary(DropLast)

#Final best model as per AIC criteria (see main text) Suggests success of bees 
#presented with flowers differing with humidity begin foraging 



#Figure 6 is generated using Excel, therefore no code is available. However plotted points are given in
#file 'PlottedPoints.csv'.
#File 'Figure 6 generation.xlsx' includes the excel object used for generation of figure 6.
#This excel object undergoes re-sizing in GNU Image Manipulation Program
#(The GIMP team, GIMP 2.10.8, www.gimp.org) and redundant tick marks for values success rate was not calculated
#at, e.g. 5, 15 etc. on the x axis are removed.

#Below is code that can be used to extract plotted the plotted points.


#Column Success.Rate gives means at each visit interval
means<- aggregate(Success.Rate~Visit:test, Learning, function(x) c(mean=mean(x)))

#Column Success.Rate gives sd at each visit interval
sd<- aggregate(Success.Rate~Visit:test, Learning, function(x) c(SD=sd(x)))
#Which can be used to calculate SEM as follows
sd$SEM<-sd$Success.Rate/sqrt(15)












